package app;


import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.InvalidPropertiesFormatException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import common.exception.AppException;
import common.msg.MsgUtil;
/**
 * Apache Poi　サンプル
 * @author isdsys
 * @apiNote 2023.02.28 created by isdsys
 */
public class AppMain {
    static Logger logger = LoggerFactory.getLogger(AppMain.class);
    
    public static Map<String, String> appConfigMap = new HashMap<String, String>() ;
    public static Map<String, String> msgMap = new HashMap<String, String>() ;

    static String logmsg;
    //デバッグ・モード
    public static boolean debug_mode= true;
    /**
     * 
     * @param args　args[0]
     * @apiNote　コマンドライン　args[0]：入力Excelファイルのパス
     * 							　args[1]: データシート名
     * 							　args[2]：出力Excelファイルのパス
     * 							　args[3]: データシート名
     * 
     */
    public static void main(String[] args) {
        
        
//        String classpath = Thread.currentThread().getContextClassLoader().getResource("").getPath();
//        logger.debug("app.AppMain::main() classpath : " + classpath);

        ExcelUtil excelUtil = new ExcelUtil();
        
        try {
            //システム定義情報取得
            getConfig();

            String sheetName 	= appConfigMap.get("dataSheetName");	//データシート名
            String inputPath 	= appConfigMap.get("inputFilePath");	//入力ファイルパス
            String outputPath 	= appConfigMap.get("inputFilePath");	//出力ファイルパス

            List<CellInfo> cellInfoList= new ArrayList<CellInfo>();

            //Excelリードセル位置設定と初期値をパラメータに設定
            if (inputPath.endsWith(".xlsx")) {
                cellInfoList.add(new CellInfo(new Position(1,0), ""));
                cellInfoList.add(new CellInfo(new Position(1,1), ""));
                cellInfoList.add(new CellInfo(new Position(1,2), ""));
            }else {
                cellInfoList.add(new CellInfo(new Position(0,0), ""));
                cellInfoList.add(new CellInfo(new Position(0,1), ""));
                cellInfoList.add(new CellInfo(new Position(0,2), ""));
            }
            
            excelUtil.read(inputPath, sheetName, cellInfoList );

//            ExcelPosition excelOutputPos = new ExcelPosition();

            //Excelライトセル位置と値をパラメータに設定
            List<CellInfo> cellInfoList2= new ArrayList<CellInfo>();
            CellInfo cellInfo =new CellInfo(new Position(1,0), "佐藤けいこ");
            cellInfoList2.add(cellInfo);
            cellInfo= new CellInfo(new Position(1,1), "女性");
            cellInfoList2.add(cellInfo);
            cellInfo = new CellInfo(new Position(1,2), "35歳");
            cellInfoList2.add(cellInfo);
            
            excelUtil.write(sheetName, cellInfoList2,outputPath);
//        } catch (EncryptedDocumentException e) {
//            // TODO 自動生成された catch ブロック
//            e.printStackTrace();
        } 
        catch (IOException e) {
            logger.error("app.AppMain::main() "+ MsgUtil.editStackTrace(e));
            
        } catch (AppException e) {
            logger.error("app.AppMain::main() "+ MsgUtil.editStackTrace(e));
        } 
    }
    /**
     * システム定義情報取得
     * @throws InvalidPropertiesFormatException
     * @throws IOException
     * @throws AppException
     */
    private static void getConfig() throws InvalidPropertiesFormatException, IOException, AppException {

        //システム定義情報取得
        appConfigMap.put("inputFilePath", "");
        appConfigMap.put("outputFilePath", "");
        appConfigMap.put("dataSheetName", "");
        String propPath = AppConst.APPCONFIG_PATH;//"cfg/cfgprop.xml";
        getProp(propPath, appConfigMap );
        
        //メッセージ定義情報取得
        msgMap.put("ERROR_FILE_IO", "");
        msgMap.put("ERROR_FILE_NOT_FOUND", "");
        msgMap.put("ERROR_PROPERTY", "");
        
        String msgPath = AppConst.MSGCONFIG_PATH;	//"cfg/msgprop.xml";
        getProp(msgPath, msgMap);
    }
    /**
     * プロパティ情報取得
     * @param fpath プロパティファイルのフルパス
     * @param propMap　プロパティのキー情報のみ設定されているマップ、本メソッドでピウロパティファイルから取得したプロパティ値を設定する
     * @throws AppException　アプリケーション例外
     */
    private static void getProp(String propPath, Map<String,String> propMap)throws AppException{

        Properties prop = new Properties();
        InputStream is;
        try {
            is = new FileInputStream(propPath);

            prop.loadFromXML(is); // is はこのメソッドが終了すると close される

            for (Iterator<String> itr = propMap.keySet().iterator(); itr.hasNext();) {
                String key = itr.next();
                if(prop.getProperty(key)==null) {
                    AppMain.logmsg = MessageFormat.format(MsgConst.ERROR_PROPERTY,key);
                    throw new AppException(AppMain.logmsg);
                }
                else {
                    //propMapに定義してあるキー値から、プロパティファイルを読み、プロパティ値を取得しpropMapに値を設定する
                    propMap.put(key, prop.getProperty(key));
                    logger.debug("app.AppMain::getProp() "+ "key= "+key + " value= "+prop.getProperty(key));
                }
            }
        } catch (FileNotFoundException e) {
            AppMain.logmsg = MessageFormat.format(MsgConst.ERROR_FILE_NOT_FOUND, propPath);
            throw new AppException(AppMain.logmsg);
            
        } catch (InvalidPropertiesFormatException e) {
            // TODO 自動生成された catch ブロック
            throw new AppException(MsgConst.ERROR_PROPERTY_FORMAT);
        } catch (IOException e) {
            // TODO 自動生成された catch ブロック
            throw new AppException(MsgConst.ERROR_FILE_IO);
        }
        
    }
    /*
    private static void getMsg(String fpath) throws AppException {
        msgMap.put("ERROR_FILE_IO", "");

        Properties prop = new Properties();
        InputStream is;
        try {
            is = new FileInputStream(fpath);

            prop.loadFromXML(is); // is はこのメソッドが終了すると close される

            for (Iterator<String> itr = msgMap.keySet().iterator(); itr.hasNext();) {
                String key = itr.next();
                System.out.println(key);
                if(prop.getProperty(key)==null) {
                    throw new AppException(key + "のプロパティ定義は、nullです。");
                }
                else {
                    propMap.put(key, prop.getProperty(key));
    //                System.out.println();
                    logger.error("AppMain::getMsg() "+ "key= "+key + "value= "+prop.getProperty(key));
    
                }
            }
        } catch (FileNotFoundException e) {
            // TODO 自動生成された catch ブロック
            throw new AppException("ファイルが存在しません。");
            
        } catch (InvalidPropertiesFormatException e) {
            // TODO 自動生成された catch ブロック
            throw new AppException("プロパティファイル書式不正。");
        } catch (IOException e) {
            // TODO 自動生成された catch ブロック
            throw new AppException("ファイルIOに失敗しました。");
        }
        
    }
    */
    /**
     * StackTrace編集
     * @param 例外オブジェクト
     * @return 編集結果
     */
/*
    static private String editStackTrace(Exception e) {
        
        StringBuffer sbf = new StringBuffer(); 
        StackTraceElement[] ste = e.getStackTrace();
        String CRLF = System.lineSeparator();

        Throwable th = e.getCause();
        int lpcnt = ste.length;
        if (th==null) {
            //最後のメッセージは出力しない
            lpcnt = ste.length-1;
        }
        sbf.append(e.getClass().getCanonicalName()).append(": ").append(e.getMessage()).append(CRLF);
        for (int i = 0; i < lpcnt; i++) {
            sbf.append("    ").append("at ").append(ste[i].getClassName()).append(".").append(ste[i].getMethodName()).append("(").append(ste[i].getFileName()).
            append(":").append(ste[i].getLineNumber()).append(")").append(CRLF);

        }
        if (th!=null) {
            sbf.append("Caused by: ").append( e.getCause().getClass().getCanonicalName()).append(":").append(e.getCause().getMessage()).append(CRLF);
            StackTraceElement[] ste2 = e.getCause().getStackTrace();
            for (int j = 0; j < (ste2.length-1); j++) {
                sbf.append("    ").append("at ").append(ste2[j].getClassName()).append(".").append(ste2[j].getMethodName()).append("(").append(ste2[j].getFileName()).
                append(":").append(ste2[j].getLineNumber()).append(")").append(CRLF); 
            }
        }
        return sbf.toString();
    }
    */
}
